/*
 * ********************************************************************************************************************
 *
 *               iFHS7.
 *              ;BBMBMBMc                  rZMBMBR              BMB
 *              MBEr:;PBM,               7MBMMEOBB:             BBB                       RBW
 *     XK:      BO     SB.     :SZ       MBM.       c;;     ir  BBM :FFr       :SSF:    ;xBMB:r   iuGXv.    i:. iF2;
 *     DBBM0r.  :D     S7   ;XMBMB       GMBMu.     MBM:   BMB  MBMBBBMBMS   WMBMBMBBK  MBMBMBM  BMBRBMBW  .MBMBMBMBB
 *      :JMRMMD  ..    ,  1MMRM1;         ;MBMBBR:   MBM  ;MB:  BMB:   MBM. RMBr   sBMH   BM0         UMB,  BMB.  KMBv
 *     ;.   XOW  B1; :uM: 1RE,   i           .2BMBs  rMB. MBO   MBO    JMB; MBB     MBM   BBS    7MBMBOBM:  MBW   :BMc
 *     OBRJ.SEE  MRDOWOR, 3DE:7OBM       .     ;BMB   RMR7BM    BMB    MBB. BMB    ,BMR  .BBZ   MMB   rMB,  BMM   rMB7
 *     :FBRO0D0  RKXSXPR. JOKOOMPi       BMBSSWBMB;    BMBB:    MBMB0ZMBMS  .BMBOXRBMB    MBMDE RBM2;SMBM;  MBB   xBM2
 *         iZGE  O0SHSPO. uGZ7.          sBMBMBDL      :BMO     OZu:BMBK,     rRBMB0;     ,EBMB  xBMBr:ER.  RDU   :OO;
 *     ,BZ, 1D0  RPSFHXR. xWZ .SMr                  . .BBB
 *      :0BMRDG  RESSSKR. 2WOMBW;                   BMBMR
 *         i0BM: SWKHKGO  MBDv
 *           .UB  OOGDM. MK,                                          Copyright (c) 2015-2020.  斯伯坦机器人
 *              ,  XMW  ..
 *                  r                                                                     All rights reserved.
 *
 * ********************************************************************************************************************
 */

package com.persagy.base.utils

import com.fasterxml.jackson.annotation.JsonInclude
import com.fasterxml.jackson.databind.ObjectMapper
import com.fasterxml.jackson.databind.PropertyNamingStrategy
import java.io.InputStream
import java.lang.reflect.Type

/**
 * SJackson 工具类
 *
 * @author  庞利祥 <sybotan@126.com>
 */
object SJacksonUtil {
    /** 命名规则 */
    val propertyNamingStrategy = PropertyNamingStrategy.LOWER_CAMEL_CASE!!

    /**
     * 转换对象到 JSON 格式字符串
     *
     * @param   obj                     被转换对象
     * @param   namingStrategy          命名规则
     * @return  JSON 格式字符串
     */
    fun toJson(obj: Any, namingStrategy: PropertyNamingStrategy? = null): String {
        val mapper = genMapper()
        mapper.setSerializationInclusion(JsonInclude.Include.NON_NULL)
        if (null  != namingStrategy) {
            mapper.propertyNamingStrategy = namingStrategy
        }

        return mapper.writeValueAsString(obj)
    }

    /**
     * 转换对象到 JSON 格式字符串，包含空字符串
     *
     * @param   obj                     被转换对象
     * @param   namingStrategy          首字母是否大写
     * @return  json格式字符串
     */
    fun toJsonAll(obj: Any, namingStrategy: PropertyNamingStrategy? = null): String {
        val mapper = genMapper()
        mapper.setSerializationInclusion(JsonInclude.Include.ALWAYS)
        if (null  != namingStrategy) {
            mapper.propertyNamingStrategy = namingStrategy
        }

        return mapper.writeValueAsString(obj)
    }

    /**
     * 将 JSON 数据解析成相应的映射对象
     *
     * @param   jsonData                要解析的JSON数据
     * @param   valueType               转换对象的类型
     * @return  解析结果
     */
    @Throws(Exception::class)
    fun <T> fromJson(jsonData: String, valueType: Class<T>, namingStrategy: PropertyNamingStrategy? = null): T? {
        val mapper = genMapper()
        if (null  != namingStrategy) {
            mapper.propertyNamingStrategy = namingStrategy
        }

        return mapper.readValue<T>(jsonData, valueType)
    }

    /**
     * 将Json数据解析成相应的映射对象
     *
     * @param   jsonData                要解析的JSON数据
     * @param   type                    Java类型
     * @return  解析结果
     */
    @Throws(Exception::class)
    fun fromJson(jsonData: String, type: Type): Any? {
        val mapper = ObjectMapper()
        return mapper.readValue(jsonData, type.javaClass)
    }

    /**
     * 将Json数据解析成相应的映射对象
     *
     * @param   input                   要解析的JSON数据输入流
     * @param   valueType               转换对象的类型
     * @param   namingStrategy          命名规则
     * @return  解析结果
     */
    @Throws(Exception::class)
    fun <T> fromJson(input: InputStream, valueType: Class<T>, namingStrategy: PropertyNamingStrategy? = null): T? {
        val mapper = genMapper()
        if (null  != namingStrategy) {
            mapper.propertyNamingStrategy = namingStrategy
        }
        return mapper.readValue(input, valueType)
    }

    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /**
     * 将Json数据解析成相应的映射对象
     *
     * @param   jsonData                要解析的JSON数据
     * @param   namingStrategy          命名规则
     * @return  解析结果
     */
    @Throws(Exception::class)
    inline fun <reified T> fromJson(jsonData: String, namingStrategy: PropertyNamingStrategy? = null): T {
        val mapper = genMapper()
        if (null  != namingStrategy) {
            mapper.propertyNamingStrategy = namingStrategy
        }
        return mapper.readValue(jsonData, T::class.java)
    }
//
//    /**
//     * 将Json数据解析成相应的映射对象
//     *
//     * @param   input                   要解析的JSON数据输入流
//     * @param   namingStrategy          命名规则
//     * @return  解析结果
//     */
//    @Throws(Exception::class)
//    inline fun <reified T> fromJson(input: InputStream, namingStrategy: PropertyNamingStrategy? = null): T {
//        val mapper = genMapper()
//        if (null  != namingStrategy) {
//            mapper.propertyNamingStrategy = namingStrategy
//        }
//        return mapper.readValue(input, T::class.java)
//    }

    /**
     * 生成 mapper 对象
     *
     * @return  mapper 对象
     */
    fun genMapper(): ObjectMapper {
        val mapper = ObjectMapper()
        mapper.propertyNamingStrategy = propertyNamingStrategy
        return mapper
    }
}
