/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2021.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.gis.json

import com.fasterxml.jackson.core.JsonGenerator
import com.fasterxml.jackson.databind.JsonSerializer
import com.fasterxml.jackson.databind.SerializerProvider
import com.persagy.gis.entity.SGisPosition

/**
 * 自定义 SGisPosition 数组对象序列化器
 *
 * @author  庞利祥 <sybotan@126.com>
 */
class SGisPositionArraySerializer : JsonSerializer<ArrayList<SGisPosition>>() {
    /** SGisPosition 对象序列化器 */
    private val positionSerializer = SGisPositionSerializer()

    /**
     * Method that can be called to ask implementation to serialize
     * values of type this serializer handles.
     *
     * @param   value           Value to serialize; can **not** be null.
     * @param   gen             Generator used to output resulting Json content
     * @param   serializers     Provider that can be used to get serializers for
     * serializing Objects value contains, if any.
     */
    override fun serialize(value: ArrayList<SGisPosition>?, gen: JsonGenerator, serializers: SerializerProvider) {
        // 如果被序列化对象为空，则直接返回
        value ?: return;

        gen.writeStartArray()

        // 依次序列化所有 SGisPosition 对象
        for (item in value) {
            positionSerializer.serialize(item, gen, serializers)
        }

        gen.writeEndArray()
    }
}