/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2021.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.gis.utils

import com.persagy.gis.entity.SGisPosition

/**
 * SGisPosition 工具对象
 *
 * @author  庞利祥 <sybotan@126.com>
 */
object SGisPositionUtil {
    /**
     * position 对象转换成 wkt 格式数据
     *
     * @param   pos     要转换的 position 对象
     * @return  wkt 格式数据
     */
    fun toWkt(pos: SGisPosition): String {
        // 如果有第三个坐标（z 轴），则生成带 Z 轴坐标值
        return if (pos.z != null) {
            "${pos.x} ${pos.y} ${pos.z}"
        } else { // 否则，生成只有 x, y 坐标值
            "${pos.x} ${pos.y}"
        }
    }

    /**
     * position 数组对象转换成 wkt 格式数据
     *
     * @param   posArr      要转换的 position 数组对象
     * @return  wkt 格式数据
     */
    fun arrayToWkt(posArr: ArrayList<SGisPosition>): String {
        // 用于生成结果的字符串 builder
        val builder = StringBuilder()

        // 依次加入所有坐标点
        for (pos in posArr) {
            // 如果已经加入过一个坐标点，则需要加入“,”分隔符
            if (builder.isNotEmpty()) {
                builder.append(", ")
            }

            builder.append(toWkt(pos))
        }

        return builder.toString()
    }

    /**
     * 二维 position 数组对象转换成 wkt 格式数据
     *
     * @param   posArr      要转换的 position 二维数组对象
     * @return  wkt 格式数据
     */
    fun array2ToWkt(posArr: ArrayList<ArrayList<SGisPosition>>): String {
        // 用于生成结果的字符串 builder
        val builder = StringBuilder()

        // 依次加入所有坐标点
        for (subArr in posArr) {
            // 如果已经加入过一个坐标点，则需要加入“,”分隔符
            if (builder.isNotEmpty()) {
                builder.append(", ")
            }

            builder.append("(${arrayToWkt(subArr)})")
        }

        return builder.toString()
    }

    /**
     * 三维 position 数组对象转换成 wkt 格式数据
     *
     * @param   posArr      要转换的 position 三维数组对象
     * @return  wkt 格式数据
     */
    fun array3ToWkt(posArr: ArrayList<ArrayList<ArrayList<SGisPosition>>>): String {
        // 用于生成结果的字符串 builder
        val builder = StringBuilder()

        // 依次加入所有坐标点
        for (subArr in posArr) {
            // 如果已经加入过一个坐标点，则需要加入“,”分隔符
            if (builder.isNotEmpty()) {
                builder.append(", ")
            }

            builder.append("(${array2ToWkt(subArr)})")
        }

        return builder.toString()
    }
}