/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.mybatis

import org.apache.ibatis.jdbc.SQL
import org.slf4j.LoggerFactory
import kotlin.collections.ArrayList
import kotlin.collections.HashMap

/**
 * SQL 构造器
 *
 * @author 庞利祥 <sybotan@126.com>
 */
open class SSqlProvider {
    /** 静态对象 */
    companion object {
        /** 日志记录器 */
        private val logger = LoggerFactory.getLogger(SSqlProvider::class.java)

        /** 驱动 */
        lateinit var driver: SAbstractDriver;
    }

    /**
     * 在数据库中插入实体
     *
     * @param argsMap         注入参数
     * @return 插入 SQL 语句
     */
    @Suppress("UNCHECKED_CAST")
    open fun insert(argsMap: Map<String, Any?>): String {
        return driver.insert(argsMap)
    }

    /**
     * 在数据库中替换实体
     *
     * @param argsMap         注入参数
     * @return 替换 SQL 语句
     */
    @Suppress("UNCHECKED_CAST")
    open fun replace(argsMap: Map<String, Any?>): String {
//        val insertSql = insert(argsMap)
        return driver.replace(argsMap)
    }

    /**
     * 根据条件删除
     *
     * @param argsMap         注入参数
     * @return 删除 SQL 语句
     */
    @Suppress("UNCHECKED_CAST")
    open fun delete(argsMap: Map<String, Any?>): String {
        return driver.delete(argsMap)
    }

    /**
     * 删除所有记录
     *
     * @param argsMap         注入参数
     * @return 删除 SQL 语句
     */
    open fun deleteAll(argsMap: Map<String, Any?>): String {
        return driver.deleteAll(argsMap)
    }

    /**
     * 根据条件更新
     *
     * @param argsMap         注入参数
     * @return 更新 SQL 语句
     */
    @Suppress("UNCHECKED_CAST")
    open fun update(argsMap: Map<String, Any?>): String {
        return driver.update(argsMap)
    }

    /**
     * 执行查询操作
     *
     * @param argsMap         注入参数
     * @return 查询 SQL 语句
     */
    @Suppress("UNCHECKED_CAST")
    fun execQuery(argsMap: Map<String, Any?>): String {
        return driver.execQuery(argsMap)
    }

    /**
     * 执行查询操作
     *
     * @param argsMap         注入参数
     * @return 查询 SQL 语句
     */
    @Suppress("UNCHECKED_CAST")
    fun execCount(argsMap: Map<String, Any?>): String {
        return driver.execCount(argsMap)
    }

    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    // 级联操作相关
    /**
     * 查询级联关系
     *
     * @param argsMap         注入参数
     * @return 查询 SQL 语句
     */
    @Suppress("UNCHECKED_CAST")
    fun relationQuery(argsMap: Map<String, Any?>): String {
        val table = argsMap["table"] as String
        val id1 = argsMap["id1"] as String
        val id2 = argsMap["id2"] as String
        val self = argsMap["self"] as Boolean
        val filter = argsMap["filter"] as String
        val idList = argsMap["idList"] as ArrayList<Any>
        val whereArgs = argsMap["whereArgs"] as HashMap<String, Any>

        var sql = object: SQL() {
            init {
                SELECT("${driver.escName(id1)} as id1, ${driver.escName(id2)} as id2")
                FROM(table)

                val builder = StringBuilder()
                for ((index, item) in idList.withIndex()) {
                    if (index > 0) {
                        builder.append(", ")
                    }
                    builder.append("#{whereArgs.args_$index}")
                    whereArgs["args_$index"] = item
                }
                if (filter != "") {
                    WHERE(filter)
                }
                WHERE("${driver.escName(id1)} IN (${builder.toString()})")
            }
        }.toString()

        /** 如果是关联自身（比如楼层连通关系，两个ID都是楼层） */
        if (self) {
            val sql1 = object: SQL() {
                init {
                    SELECT("${driver.escName(id2)} as id1, ${driver.escName(id1)} as id2")
                    FROM(table)

                    val builder = StringBuilder()
                    for ((index, item) in idList.withIndex()) {
                        if (index > 0) {
                            builder.append(", ")
                        }
                        builder.append("#{whereArgs.args_$index}")
                        whereArgs["args_$index"] = item
                    }
                    if (filter != "") {
                        WHERE(filter)
                    }
                    WHERE("${driver.escName(id2)} IN (${builder.toString()})")
                }
            }.toString()

            sql = "$sql \n UNION \n $sql1"
        }

        logger.debug("SQL= $sql")
        return sql
    }
}
